function sig_u = Renyi_entropy(s,sgm,al,t1)
%%% find the time-varying sigma by Renyi entropy
%%% for SSO
% output sig_u:   the time-varying parameter
% input  x:   the signal to be analyzed
%      sgm:   the possible range of the parameter sigma
%       t1:   a parameter to determine the length for integration
%       al:   a constant, al>2
%       mu:   the wavelet paramter

N = length(s);
n_sgm = length(sgm);
Re0 = zeros(n_sgm,N);
rb2 = zeros(1,N);
rb1 = rb2;

alfa=2*al;

for k = 1:n_sgm
    ci = sgm(k);
    
    STFT = stft_may12(s,ci);

    Ds = abs(STFT);
    Ds = [fliplr(Ds(:,1:t1)) Ds fliplr(Ds(:,end-t1+1:end))]; %padding
    for i=t1+1:N-t1
        rb1(i-t1)=sum(sum(Ds(:,(i-t1):(i+t1)).^alfa));
        rb2(i-t1)=(sum(sum(Ds(:,(i-t1):(i+t1))).^2)).^al;
    end
    for i=1:N
        Re0(k,i)=1/(1-al)*log2((rb1(:,i)./rb2(:,i))+eps);
    end
end

[x,y] = min(abs(Re0),[],1);
sig_u = sgm(y);          %ҳÿʱСentropyֵӦsigmaֵ
end



function STFT = stft_may12(s,ci)
%%% sst2 : computes the STFT of a signal and different versions of synchrosqueezing/reassignment.
%   Uses a Gaussian window.
%
% INPUTS:  
%   s: real or complex signal, must be of length 2^N
%   gamma: threshold
%   sigma: window parameter
%   ft: frequency bins
%   bt: time bins
% OUTPUTS:   
%   STFT: the short-time Fourier transform


% checking length of signal
n = length(s);
nv = log2(n);
if mod(nv,1)~=0
    warning('The signal is not a power of two, truncation to the next power');
    s = s(1:2^floor(nv));
end
n = length(s);
s = s(:);


% Optional parameters
if nargin<5
   bt = 1:n;
   ft = 1:n/2;
end
nb = length(bt);
neta = length(ft);


% Padding
sleft = flipud(s(2:n/2+1));
sright = flipud(s(end-n/2:end-1));
x = [sleft; s; sright];
clear xleft xright;

sigma = ci;

% Window definition
t = linspace(-0.5,0.5,n);t=t';
g = (1/((2*pi).^(1/2)*sigma))*exp(-1/(2*sigma^2)*t.^2);

% Initialization
STFT = zeros(neta,nb);

%% Computes STFT and reassignment operators
df = ft(2)-ft(1);
% db = bt(2)-bt(1);
for b=1:nb
	% STFT, window g
    tmp = (fft(x(bt(b):bt(b)+n-1).*g))/n;
    vg = tmp(ft);  
    % Storing STFT
    STFT(:,b) = vg.* exp(1i*pi*(ft-1)'); % compensates the tranlation 1/2 of s
        
end
STFT = df*STFT;
end


