function [Frequency,Function,Function_gt,rfts,Function_sin_model] = SSO_recov(signal,sigma,gamma,cr_gt) 
%%% This code calculate the frequency curves and the recoverd functions by SSO and ASSO (with the true chirp rate and estimated chirp rate, respectively) 
%%% "signal" is the input signal 
%%% "sigma" is the input parameter for Gaussian function,"WindowWidth" is the width 'a' of window function h(x)
%%% "gamma" is an input parameter 
%%% cr_gt   ground truth chirp rate
%%% Outputs:  "Frequency,Function,Function_gt,rfts,Function_sin_model"

N = length(signal);
%% Parameters (They need to be adjusted in different applications)
Nb = 64; % bound

j = [-N/2+Nb:N/2-Nb];
TimeVariable_0 = j/N;
ThetaVariable = [0:1*N-1]/(2*N); % Thelta points （this should be on （0,pi）according to the SSO demo）

%% Padding, symmetric  M = 2*N
sleft = fliplr(signal(2:N/2+1));
sright = fliplr(signal(end-N/2:end-1));
x = [sleft signal sright];
clear xleft xright;
M = length(x);   % M = 2*N;is the length of the padded signal
n1 = length(sleft);


%% Calculate \TauF(t,thelta) named SSOmatrix by SSO
 SSOmatrix = zeros(N,length(ThetaVariable));
for k = 1:N
    sigm0 = sigma(k);
    gf = 1/(sqrt(2*pi)*sigm0) * exp(-TimeVariable_0.^2/(2*sigm0^2))/N;   % Gaussian function
    for n = 1:length(ThetaVariable)
        x_trun = x(k+n1-N/2+Nb:k+n1+N/2-Nb);
        SSOmatrix(k,n) = sum(gf.*x_trun.*exp(1i*2*pi*ThetaVariable(n)*j));
    end
end


%% find the ridge with forward/backward search
Tx = SSOmatrix';
ccur = ones(1,N);
Tx0 = Tx;
Tx0(:,1:Nb) = 0; Tx0(:,end-Nb+1:end) = 0;
[mxm, c1] = max(max(abs(Tx0)));
[mxm, d1] = max(max(abs(Tx0')));
ccur(c1) = d1;

mstep = 3; % the maximum frequency hopping for one component

flag = 1;  % forward search
while c1 < N && flag       
    bt = c1+1;
    TauVector=Tx(:,bt); % fix t, \TauF(t,thelta)
    tep = abs(TauVector);
    tep = tep(:);
    tep = tep';
    tep1 = tep;%/max(tep);
    l_tep1 = [0,0,tep1(4:end-1),0,0];
    r_tep1 = [0,0,tep1(2:end-3),0,0];
    m_tep1 = [0,0,tep1(3:end-2),0,0];
    a_1 = find(((m_tep1>l_tep1)+(m_tep1>r_tep1))==2 & tep>gamma*mxm);
    da_1 = a_1-d1;
    [mda, ix2] = min(abs(da_1));
    if mda<mstep
        ccur(bt) = a_1(ix2);
        d1 = a_1(ix2);
    else
        flag=0;
    end
    c1 = bt;
end
 
[mxm, c1] = max(max(abs(Tx0)));
[mxm, d1] = max(max(abs(Tx0')));
ccur(c1) = d1;
flag = 1;  % backward search
while c1 > 1 && flag       
    bt = c1-1;
    TauVector=Tx(:,bt); % fix t, \TauF(t,thelta)
    tep = abs(TauVector);
    tep = tep(:);
    tep = tep';
    tep1 = tep;%/max(tep);
    l_tep1 = [0,0,tep1(4:end-1),0,0];
    r_tep1 = [0,0,tep1(2:end-3),0,0];
    m_tep1 = [0,0,tep1(3:end-2),0,0];
    a_1 = find(((m_tep1>l_tep1)+(m_tep1>r_tep1))==2 & tep>gamma*mxm);
    da_1 = a_1-d1;
    [mda, ix2] = min(abs(da_1));
    if mda<mstep
        ccur(bt) = a_1(ix2);
        d1 = a_1(ix2);
    else
        flag=0;
    end
    c1 = bt;
end
        
    
%% Recover Aj(t)cos(2*pi*phi(t));
Frequency = (ccur)/2;   %\20s
% 
sm_tf = smooth(Frequency,20,'rlowess');
sm_tf = sm_tf';
rft = N*(diff([sm_tf(1) sm_tf ]) + diff([sm_tf sm_tf(N)]))/2;   %% estimation of chirp rate
rfts = smooth(rft,20,'rlowess');
   
for kk = 2:length(Frequency)-1     %  fitting 
    lb = max([kk-3,1]);
    rb = min([kk+3,length(Frequency)]);
    wrb = min([kk-lb,rb-kk]);
    xtmp = Frequency(kk-wrb:kk+wrb);
    ti_x = [-wrb:wrb]/length(Frequency);
    pppp = polyfit(ti_x,xtmp,1);
    rft(kk) = pppp(1);
end  
rft(1) = rft(2); rft(length(Frequency)) = rft(length(Frequency)-1);
rfts1 = smooth(rft,20,'rlowess');   
    

Function = zeros(1,N);
Function_gt = zeros(1,N);
Function_sin_model = zeros(1,N);
for kk = 1:N
    test(kk) = 2*pi*sigma(kk)^2*rft(kk);
    Function(kk) = 2*real(sqrt(1-1i*2*pi*sigma(kk)^2*rfts(kk))*Tx(ccur(kk),kk));
    Function_gt(kk) = 2*real(sqrt(1-1i*2*pi*sigma(kk)^2*cr_gt)*Tx(ccur(kk),kk));
    Function_sin_model(kk) = 2*real(Tx(ccur(kk),kk));
end





