% ASSO Toolbox v1.0
% This is an example for the LFM signal
% Lin Li, October 2019
% Copyright (c) 2019 Xidian University
% This program can not be used for commercialization without the authorization of its author(s). 

% If you use this toolbox for research, please must cite the following papers:
% [1] Lin Li, Charles K. Chui and Qingtang Jiang, "Direct Signal Separation via Extraction of Local Frequencies With Adaptive Time-Varying Parameters," in IEEE Transactions on Signal Processing, vol. 70, pp. 2321-2333, 2022.
% [2] Lin Li, Haiyan Cai, and Qingtang Jiang, "Adaptive synchrosqueezing transform with a time-varying parameter for non-stationary signal separation," Applied and Computational Harmonic Analysis, 49(3), (2020): 1075–1106.
% [3] Lin Li, Haiyan Cai, Hongxia Han, Qingtang Jiang, and Hongbing Ji, "Adaptive short-time Fourier transform and synchrosqueezing transform for non-stationary signal separation,"Signal Processing, 166 (2020), 107231.
% [4] Charles K. Chui, Qingtang Jiang, Lin Li, Jian Lu, "Analysis of an adaptive short-time Fourier transform-based multicomponent signal separation method derived from linear chirp local approximation," Journal of Computational and Applied Mathematics, Volume 396, 2021, 113607

% For any comment or bug report, please send e-mail to lilin@xidian.edu.cn or jiangq@umsl.edu. 

clear;
close all;
clc;

%%%%%%% October 2019
N = 512;
Fs = N;
Ts = 1/Fs;
ti = [0:N-1]/Fs; % s
fi = ti.*Fs;    % Hz
Tp = ti(2)-ti(1); % s

c1 = 17;
r1 = 37;    
A1= 1;
x1 = A1*cos(2*pi*(c1*ti+r1/2*ti.^2));
f1 = c1+r1*ti;

c2 = 31;
r2 = 171;   %171
A2= 1;
x2 = A2*cos(2*pi*(c2*ti+r2/2*ti.^2));
f2 = c2+r2*ti;

s = x1+x2;  % two-LFM signal

s = x1;  %mono-LFM signal
cr_gt = r1;

% 
% figure;
% plot(ti,s,'k-','linewidth',1);
% xlabel('Time (s)','FontSize',20);
% ylabel('Amplitude','FontSize',20);
% axis([0 1 -3 3]);
% set(gca,'FontSize',20)
% print(gcf,'-dpng ','waveform_tow_chirp.png');


sgm1 = 1/sqrt(2*pi*r1);  % optimal for x1
sgm2 = 1/sqrt(2*pi*r2); % optimal for x2

SSOmatrix = SSO_matrix_calc(s,sgm1); 
figure;
imagesc([0,1],[0 1],abs(SSOmatrix));
ylabel('\it{t}','FontSize',20);
xlabel('\theta','FontSize',20);
set(gca,'FontSize',20)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   global time-varying Parameter
sgm = [0.02:0.001:0.05];
t1 = 20;
al = 2.5;
sig_u = Renyi_entropy(s,sgm,al,t1);
% figure;
% plot(sig_u);
sig_global = median(sig_u);  %


signal =s; 
gamma = 0.01;
[freq1, func1,func1_gt,cpr1,func1_regu] = SSO_recov(signal,sig_global*ones(1,N),gamma,cr_gt);


figure;
plot(ti,s,'b-','linewidth',0.5);
hold on;
plot(ti,func1,'r--','linewidth',1);
legend ('Groud truth','Recovered signal');
set(legend,'FontName','Times New Roman')
xlabel('Time (s)','FontSize',20);
ylabel('Amplitude','FontSize',20);
set(gca,'FontSize',20);


figure;
plot(ti,abs(s-func1_regu),'b--','linewidth',0.5);
hold on;
plot(ti,abs(s-func1),'r-','linewidth',1);
legend ('Error with sinusoidal-model','Error with linear-chirp model','location','north');
set(legend,'FontName','Times New Roman')
xlabel('Time (s)','FontSize',20);
ylabel('Amplitude','FontSize',20);
axis([0.2 0.8 0 0.095]);
set(gca,'FontSize',20);



figure;
plot(ti,s,'b-','linewidth',0.5);
hold on;
plot(ti,func1_gt,'r--','linewidth',1);
legend ('Groud truth','Recovered signal');
set(legend,'FontName','Times New Roman')
xlabel('Time (s)','FontSize',20);
ylabel('Amplitude','FontSize',20);
set(gca,'FontSize',20);


figure;
plot(ti,abs(s-func1_regu),'b--','linewidth',0.5);
hold on;
plot(ti,abs(s-func1_gt),'r-','linewidth',1);
legend ('Error with sinusoidal-model','Error with linear-chirp model','location','north');
set(legend,'FontName','Times New Roman')
xlabel('Time (s)','FontSize',20);
ylabel('Amplitude','FontSize',20);
axis([0.2 0.8 0 0.095]);
set(gca,'FontSize',20);

